/* SyncPDFDate - Adjust file timestamp to match date embedded in PDF

   FIXME to offer manual adjust

   Copyright (c) 2020-2022 Steven Levine and Associates, Inc.
   All rights reserved.

   2020-02-05 SHL Baseline - clone from elsewhere
   2020-03-30 SHL Say why adjust bypassed
   2020-07-09 SHL Sync with tempalates - AskYNQ
   2020-07-09 SHL Support iso date with hh:mm only
   2021-01-11 SHL Support gmt iso date 2021-01-04T09:06Z
   2021-09-09 SHL Sync with tempalates - AskYNQ
   2022-03-13 SHL Sync with tempalates - AskYNQ
   2022-03-13 SHL Offer to run SyncDateToName
   2022-04-03 SHL Offer to run SyncDateToName even if date matches
   2022-04-19 SHL Use Quote for shell
   2022-05-28 SHL Correct missing QuoteForShell
*/

signal on Error
signal on Failure name Error
signal on Halt
signal on NotReady name Error
signal on NoValue name Error
signal on Syntax name Error

gVersion = '0.1 2022-05-28'

Globals = 'gArgList. gCmdName gDbgLvl gEnv gVerbose gVersion'

call Initialize

Main:
  parse arg cmdLine
  call ScanArgs cmdLine
  drop cmdLine

  numeric digits 14

  say
  call DoWildArgs

  exit

/* end main */

/*=== DoWildArgs() Process wildcarded argument list ===*/

DoWildArgs: procedure expose (Globals)

  do argNum = 1 to gArgList.0
    wildCard = gArgList.argNum
    wildCard = strip(wildCard,, '"')
    call DbgMsg 'Searching for' wildCard

    /* Format options F: files D: directories S: recurse */
    /* TL: YYYY-MM-DD HH:MM:SS Size ADHRS Name */
    call SysFileTree wildCard, 'fileList', 'FTL'
    if RESULT \= 0 then
      call Die 'SysFileTree' wildCard 'failed'
    do fileNum = 1 to FileList.0
      call DoOneFile FileList.fileNum
    end /* fileNum */
  end /* argNum */

  return

/* end DoWildArgs */

/*=== DoOneFile(fileinfo) Process one file ===*/

DoOneFile: procedure expose (Globals)

  parse arg fileInfo
  parse var fileinfo fileDate fileTime fileBytes fileAttrib pathName
  pathName = strip(pathName)

  i = lastpos('\', pathName)

  if i = 0 then
    call Die 'Can not find \ in' pathname

  filename = substr(pathname, i + 1)

  i = lastpos('.', filename)
  if i = 0 then
    call Die 'Can not find . in' filename

  ext = substr(filename, i)

  basename = left(filename, i - 1);

  call DbgMsg 2, 'basename' basename

  shortname = basename || ext

  numeric digits 14

  say 'Checking' pathName

  call stream pathName, 'C', 'OPEN READ'

  ymd = ''
  hms = ''

  do while lines(pathName) > 0

    line = linein(pathName)

    /* Look for
      /ModDate (D:19981109183526)
      /ModDate(D:20080807 090635-07'00')
      /CreationDate(D:19981106232806)
      <xmp:ModifyDate>2013-06-03T13:01:15-04:00</xmp:ModifyDate>
      <xmp:CreateDate>2013-04-01T10:01:07-04:00</xmp:CreateDate>

    */
    if pos('Date', line) = 0 then
      iterate

    /* Check modify date before create date in case both on same line */
    i = pos('/ModDate', line)
    if i = 0 then
      i = pos('/CreationDate', line)
    if i > 0 then
      iso = 0
    else do
      iso = 1
      i = pos('<xmp:ModifyDate>', line)
      if i = 0 then
	i = pos('<xmp:CreateDate>', line)
      if i = 0 then
	iterate
    end

    call DbgMsg 2, 'line' line

    if iso then do
      parse var line =(i) . '>'tstamp'<' .
      /* 2013-04-01T10:01:07:22-04:00 */
      /* 2013-04-01T10:01:07-04:00 */
      /* 2021-01-04T09:06Z */
      call DbgMsg 'iso timestamp "' || tstamp || '"'
      newYmd = left(tstamp, 10)
      newHms = strip(substr(tstamp, 12, 8))
      if substr(newHms, 6, 1) == '-' then
	newHms = left(newHms, 5)	/* chop TZ */
      newHMS = strip(newHMS, 'T', 'Z')
      if length(newHMS) = 5 then
	newHMS = newHMS || ':00'	/* Add :00 */
    end
    else do
      parse var line =(i) . '(D:'tstamp')' .
      call DbgMsg 'ymd timestamp' tstamp
      if \ datatype(left(tstamp, 14), 'W') then
	iterate
      newYmd = translate('ABCD-EF-GH', left(tstamp, 8), 'ABCDEFGH')
      newHms = translate('AB:CD:EF', substr(tstamp, 9, 6), 'ABCDEF')
    end

    call DbgMsg 'newYmd' newYmd 'newHms' newHms

    if newYmd > ymd then do
      ymd = newYmd
      hms = newHms
    end
    if newHms > hms then
      hms = newHms

  end /* while */

  call stream pathName, 'C', 'CLOSE'

  call DbgMsg 'ymd' ymd 'hms' hms 'fileDate' fileDate 'fileTime' fileTime

  /* If date adjust needed, prompt to change */

  if ymd == '' then do
    call AskYNQ 'Cannot extract pdf timestamp - try SyncDateToName'
    if RESULT >= 2 then
      exit
    if RESULT = 0 then
       call SyncDateToName QuoteForShell(pathName)
  end
  else if ymd == fileDate then do
    todayYMD = translate('ABCD-EF-GH', date('S'), 'ABCDEFGH')
    /* Some PDFs marked modified today because they are generated on demand
       We prefer to override with date encoded in file name
    */
    if ymd < todayYMD then do
      say 'File timestamp matches pdf timestamp' ymd hms
    end
    else do
      call AskYNQ 'File timestamp matches pdf timestamp' ymd hms 'override with SyncDateToName'
      if RESULT >= 2 then
	exit
      if RESULT = 0 then
	 call SyncDateToName QuoteForShell(pathName)
    end
  end
  else do
    say
    say shortname
    say 'File timestamp is        ' fileDate fileTime
    say 'Internal pdf timestamp is' ymd hms
    call AskYNQ 'Change file timestamp to to pdf timestamp'
    if RESULT >= 2 then
      exit
    if RESULT = 0 then do
       err = SysSetFileDateTime(pathName, ymd, hms)
       if err \= 0 then
	 call Die 'Can not set date'
    end
    say
  end

  return

/* end DoOneFile */

/*=== Initialize() Initialize globals ===*/

Initialize: procedure expose (Globals)
  call SetCmdName
  call LoadRexxUtil
  gEnv = 'OS2ENVIRONMENT'
  return

/* end Initialize */

/*=== ScanArgsInit() ScanArgs initialization exit routine ===*/

ScanArgsInit: procedure expose (Globals) cmdTail swCtl keepQuoted
  /* Preset defaults */
  gDbgLvl = 0				/* Display debug messages */
  gVerbose = 0				/* Verbose messages */
  gArgList.0 = 0			/* Reset arg count */
  keepQuoted = 1			/* Set to 1 to keep arguments quoted */
  return

/* end ScanArgsInit */

/*=== ScanArgsSwitch() ScanArgs switch option exit routine ===*/

ScanArgsSwitch: procedure expose (Globals) curSw curSwArg
  select
  when curSw == 'd' then
    gDbgLvl = gDbgLvl + 1
  when curSw == 'h' | curSw == '?' then
    call ScanArgsHelp
  when curSw == 'v' then
    gVerbose = 1
  when curSw == 'V' then do
    say gCmdName gVersion
    exit
  end
  otherwise
    call ScanArgsUsage 'switch '''curSw''' unexpected'
  end /* select */

  return

/* end ScanArgsSwitch */

/*=== ScanArgsArg() ScanArgs argument option exit routine ===*/

ScanArgsArg: procedure expose (Globals) curArg
  i = gArgList.0 + 1
  gArgList.i = curArg
  gArgList.0 = i
  return

/* end ScanArgsArg */

/*=== ScanArgsTerm() ScanArgs scan end exit routine ===*/

ScanArgsTerm: procedure expose (Globals)
  if gArgList.0 = 0 then do
    gArgList.1 = '*.pdf'
    gArgList.0 = 1
  end
  return

/* end ScanArgsTerm */

/*=== ScanArgsHelp() Display ScanArgs usage help exit routine ===*/

ScanArgsHelp:
  say
  say 'Adjust file timestamp to match date embedded in file name.'
  say
  say 'Usage:' gCmdName '[-d] [-h] [-v] [-V] [-?] [filename]...'
  say
  say '  -d         Enable debug logic, repeat for more verbosity'
  say '  -h -?      Display this message'
  say '  -v         Enable verbose output'
  say '  -V         Display version number and quit'
  say
  say '  filename   Files to adjust, wildcards OK, defaults to *.pdf'
  exit 255

/* end ScanArgsHelp */

/*=== ScanArgsUsage(message) Report Scanargs usage error exit routine ===*/

ScanArgsUsage:
  parse arg msg
  say
  if msg \== '' then
    say msg
  say 'Usage:' gCmdName '[-d] [-h] [-v] [-V] [-?] [filename]...'
  exit 255

/* end ScanArgsUsage */

/*==============================================================================*/
/*=== SkelRexxFunc standards - Delete unused - Move modified above this mark ===*/
/*==============================================================================*/

/*=== AskYNQ([prompt][, noskip]) returns 0=Yes, 1=No, 2=Quit, skips line unless noskip ===*/

AskYNQ: procedure
  parse arg msg, noskip

  /* Take focus or fail if cannot match window title - assumes running 4OS2 */
  signal off Error
  /* Map 1st left bracket to wild card - [pid] seems to confuse activate */
  '@if defined _WINTITLE activate "%@replace[[,*,%_WINTITLE]"'
  signal on Error

  /* Skip line unless suppressed by noskip arg - any non-zero value requests noskip */
  if noskip = '' | noskip = 0 then
    call lineout 'STDERR', ''

  if msg == '' then
    msg = 'Continue'
  call charout 'STDERR', msg '(y/n/q) ? '
  do forever
    key = translate(SysGetKey('NOECHO'))
    if key == 'Y' | key == 'N' then do
      call lineout 'STDERR', key
      if key == 'Y' then
	ynq = 0
      else
	ynq = 1
      leave
    end
    if key == 'Q' | c2x(key) == '1B' then do
      call lineout 'STDERR', ''
      ynq = 2
      leave
    end
  end /* forever */
  return ynq

/* end AskYNQ */

/*=== DbgMsg([minLvl, ]message,...) Optionally write multi-line message to STDERR ===*/

/**
 * Write message if gDbgLvl >= minLvl
 * @param minLvl defaults to 1 if omitted
 * @returns true if message written
 */

DbgMsg: procedure expose (Globals)
  minLvl = arg(1)
  if datatype(minLvl, 'W') then
    start = 2
  else do
    minLvl = 1
    start = 1
  end
  if gDbgLvl >= minLvl then do
    do i = start to arg()
      msg = arg(i)
      if msg \== '' then
	msg = ' *' msg
      call lineout 'STDERR', msg
    end
  end
  return gDbgLvl >= minLvl

/* end DbgMsg */

/*=== QuoteForShell(string) Quote string if it contains shell specials ===*/

QuoteForShell: procedure expose (Globals)

  parse arg s

  /* Assume 4OS2 */
  specials = '<>|&+'			/* 2014-01-06 SHL add plus */
  if symbol('gShell') == 'VAR' then do
    if gShell == 'cmd' then
      specials = '<>|&^'
  end

  /* Quote if non-printable or shell special or 4OS2 special
   * Assume %+ is &
   */
  if verify(s, xrange('21'x, '7e'x)) \= 0 |,
     verify(s, specials, 'M') \= 0 then
    s = '"' || s || '"'

  return s

/* end QuoteForShell */

/*==========================================================================*/
/*=== SkelRexx standards - Delete unused - Move modified above this mark ===*/
/*==========================================================================*/

/*=== Die([message,...]) Write multi-line message to STDERR and die ===*/

Die:
  call lineout 'STDERR', ''
  do i = 1 to arg()
    call lineout 'STDERR', arg(i)
  end
  call lineout 'STDERR', gCmdName 'aborting at line' SIGL || '.'
  call beep 200, 300
  call SysSleep 2
  exit 254

/* end Die */

/*=== Error() Set gErrCondition; report to STDOUT; trace and exit or return if called ===*/

Error:
  say
  parse source . . cmd
  gErrCondition = condition('C')
  say gErrCondition 'signaled at line' SIGL 'of' cmd || '.'
  if condition('D') \== '' then
    say 'REXX reason =' condition('D') || '.'
  if gErrCondition == 'SYNTAX' & symbol('RC') == 'VAR' then
    say 'REXX error =' RC '-' errortext(RC) || '.'
  else if symbol('RC') == 'VAR' then
    say 'RC =' RC || '.'
  say 'Source =' sourceline(SIGL)

  if condition('I') \== 'CALL' | gErrCondition == 'NOVALUE' | gErrCondition == 'SYNTAX' then do
    trace '?A'
    say 'Enter REXX commands to debug failure.  Press enter to exit script.'
    nop
    if symbol('RC') \== 'VAR' then
      RC = 255
    exit RC
  end

  return

/* end Error */

/*=== Halt() Report HALT condition to STDOUT and exit ===*/

Halt:
  say
  parse source . . cmd
  say condition('C') 'signaled at' cmd 'line' SIGL || '.'
  say 'Source =' sourceline(SIGL)
  say 'Sleeping for 2 seconds...'
  call SysSleep 2
  exit 253

/* end Halt */

/*=== LoadRexxUtil() Load RexxUtil functions ===*/

LoadRexxUtil:
  if RxFuncQuery('SysLoadFuncs') then do
    call RxFuncAdd 'SysLoadFuncs', 'REXXUTIL', 'SysLoadFuncs'
    if RESULT then
      call Die 'Cannot load SysLoadFuncs.'
    call SysLoadFuncs
  end
  return

/* end LoadRexxUtil */

/*=== ScanArgs(cmdLine) Scan command line ===*/

ScanArgs: procedure expose (Globals)

  /* Calls user exits to process arguments and switches */

  parse arg cmdTail
  cmdTail = strip(cmdTail)

  call ScanArgsInit

  /* Ensure optional settings initialized */
  if symbol('SWCTL') \== 'VAR' then
    swCtl = ''				/* Switches that take args, append ? if optional */
  if symbol('KEEPQUOTED') \== 'VAR' then
    keepQuoted = 0			/* Set to 1 to keep arguments quoted */

  /* Scan */
  curArg = ''				/* Current arg string */
  curSwList = ''			/* Current switch list */
  /* curSwArg = '' */			/* Current switch argument, if needed */
  noMoreSw = 0				/* End of switches */

  do while cmdTail \== '' | curArg \== '' | curSwList \== ''

    /* If arg buffer empty, refill */
    if curArg == '' then do
      qChar = left(cmdTail, 1)		/* Remember quote */
      if \ verify(qChar,'''"', 'M') then
	parse var cmdTail curArg cmdTail	/* Not quoted */
      else do
	/* Arg is quoted */
	curArg = ''
	do forever
	  /* Parse dropping quotes */
	  parse var cmdTail (qChar)quotedPart(qChar) cmdTail
	  curArg = curArg || quotedPart
	  /* Check for escaped quote within quoted string (i.e. "" or '') */
	  if left(cmdTail, 1) \== qChar then do
	    cmdTail = strip(cmdTail)	/* Strip leading whitespace */
	    leave			/* Done with this quoted arg */
	  end
	  curArg = curArg || qChar	/* Append quote */
	  if keepQuoted then
	    curArg = curArg || qChar	/* Append escaped quote */
	  parse var cmdTail (qChar) cmdTail	/* Strip quote */
	end /* do forever */
	if keepQuoted then
	  curArg = qChar || curArg || qChar	/* requote */
      end /* if quoted */
    end /* if curArg empty */

    /* If switch buffer empty, refill */
    if curSwList == '' & \ noMoreSw then do
      if left(curArg, 1) == '-' & curArg \== '-' then do
	if curArg == '--' then
	  noMoreSw = 1
	else
	  curSwList = substr(curArg, 2)	/* Remember switch string */
	curArg = ''			/* Mark empty */
	iterate				/* Refill arg buffer */
      end /* if switch */
    end /* if curSwList empty */

    /* If switch in progress */
    if curSwList \== '' then do
      curSw = left(curSwList, 1)	/* Next switch */
      curSwList = substr(curSwList, 2)	/* Drop from pending */
      /* Check switch allows argument, avoid matching ? */
      if pos(curSw, translate(swCtl,,'?')) \= 0 then do
	if curSwList \== '' then do
	  curSwArg = curSwList		/* Use rest of switch string for switch argument */
	  curSwList = ''
	end
	else if curArg \== '' & left(curArg, 1) \== '-' then do
	  curSwArg = curArg		/* Arg string is switch argument */
	  curArg = ''			/* Mark arg string empty */
	end
	else if pos(curSw'?', swCtl) = 0 then
	  call ScanArgsUsage 'Switch "-' || curSw || '" requires an argument'
	else
	  curSwArg = ''			/* Optional arg omitted */
      end

      call ScanArgsSwitch		/* Passing curSw and curSwArg */
      drop curSwArg			/* Must be used by now */
    end /* if switch */

    /* If arg */
    else if curArg \== '' then do
      noMoreSw = 1
      call ScanArgsArg			/* Passing curArg */
      curArg = ''
    end

  end /* while not done */

  call ScanArgsTerm

  return

/* end ScanArgs */

/*=== SetCmdName() Set gCmdName to short script name ===*/

SetCmdName: procedure expose (Globals)
  parse source . . cmd
  cmd = filespec('N', cmd)		/* Chop path */
  c = lastpos('.', cmd)
  if c > 1 then
    cmd = left(cmd, c - 1)		/* Chop extension */
  gCmdName = translate(cmd, xrange('a', 'z'), xrange('A', 'Z'))	/* Lowercase */
  return

/* end SetCmdName */

/* The end */
